/* Classic Ladder Project */
/* Copyright (C) 2001 Marc Le Douarain */
/* mavati@club-internet.fr */
/* http://www.multimania.com/mavati/classicladder */
/* February 2001 */
/* Last update : 17 April. 2001 */

/* This library is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU Lesser General Public */
/* License as published by the Free Software Foundation; either */
/* version 2.1 of the License, or (at your option) any later version. */

/* This library is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* Lesser General Public License for more details. */

/* You should have received a copy of the GNU Lesser General Public */
/* License along with this library; if not, write to the Free Software */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#define NBR_RUNGS 100
#define NBR_BITS 500
#define NBR_TIMERS 10
#define NBR_MONOSTABLES 10

#define TIME_REFRESH_RUNG_MS 100
#define TIME_BASE_MINS 60000
#define TIME_BASE_SECS 1000
#define TIME_BASE_100MS 100

/* numbers of blocks in a rung */
#define RUNG_WIDTH 10
#define RUNG_HEIGHT 6

/* size in pixels */
#define BLOCK_WIDTH 32
#define BLOCK_HEIGHT 32
/* offsets in pixels */
#define OFFSET_X 4
#define OFFSET_Y 6

/* elements in the rungs */
#define ELE_FREE 0
#define ELE_INPUT 1
#define ELE_INPUT_NOT 2
#define ELE_RISING_INPUT 3
#define ELE_FALLING_INPUT 4
#define ELE_CONNECTION 9
#define ELE_TIMER 10
#define ELE_MONOSTABLE 11
#define ELE_OUTPUT 50
#define ELE_OUTPUT_NOT 51
#define ELE_OUTPUT_SET 52
#define ELE_OUTPUT_RESET 53
#define ELE_OUTPUT_JUMP 54
#define ELE_UNUSABLE 99 /* for complex elements using
                        many blocks : only one block
                        is "alive" */

#define VAR_MEM_BIT 00
#define VAR_TIMER_DONE 10
#define VAR_TIMER_RUNNING 11
#define VAR_MONOSTABLE_RUNNING 20
//#define VAR_PHYS_INPUT 50
//#define VAR_PHYS_OUTPUT 60

#define BASE_MINS 0
#define BASE_SECS 1
#define BASE_100MS 2

typedef struct StrElement
{
    short int Type;
    char ConnectedWithTop;
    int VarType;
    int VarNum;     /* or NumRung (for jump), NumTimer, NumMonostable,... */

    char DynamicInput;
    char DynamicState;
    char DynamicVarBak; /* used for rising/falling edges */
    char DynamicOutput;
}StrElement;

#define LGT_LABEL 10
#define LGT_COMMENT 30
typedef struct StrRung
{
    int Used;
    int PrevRung;
    int NextRung;
    char Label[LGT_LABEL];
    char Comment[LGT_COMMENT];
    StrElement Element[RUNG_WIDTH][RUNG_HEIGHT];
}StrRung;

typedef struct StrTimer
{
    int Preset;
    int Value;
    int Base;
    char DisplayFormat[10];
    char InputEnable;
    char OutputDone;
    char OutputRunning;
}StrTimer;

typedef struct StrMonostable
{
    int Preset;
    int Value;
    int Base;
    char DisplayFormat[10];
    char Input;
    char InputBak;
    char OutputRunning;
}StrMonostable;
